@@ -27,7 +27,8 @@ import { ChipModule } from 'primeng/chip'
 import { EntityLinkComponent } from '../entity-link/entity-link.component'
 import { DividerModule } from 'primeng/divider'
 import { HostDataSourceLabelComponent } from '../host-data-source-label/host-data-source-label.component'
-import { MessageModule } from 'primeng/message'
+import { ProgressSpinnerModule } from 'primeng/progressspinner'
+import { MessagesModule } from 'primeng/messages'
 
 describe('HostTabComponent', () => {
     let component: HostTabComponent
@@ -54,7 +55,8 @@ describe('HostTabComponent', () => {
                 ToggleButtonModule,
                 TreeModule,
                 TagModule,
-                MessageModule,
+                MessagesModule,
+                ProgressSpinnerModule,
             ],
             declarations: [
                 DhcpClientClassSetViewComponent,
@@ -82,7 +84,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should display v4 host information', () => {
-        const host = {
+        const host: Partial<Host> = {
             id: 1,
             hostIdentifiers: [
                 {
@@ -106,6 +108,12 @@ describe('HostTabComponent', () => {
                     nextServer: '192.0.2.2',
                     serverHostname: 'my-server',
                     bootFileName: '/tmp/bootfile1',
+                    hostname: 'mouse.example.org',
+                    ipReservations: [
+                        {
+                            address: '192.0.2.123',
+                        },
+                    ],
                 },
                 {
                     appId: 2,
@@ -114,6 +122,12 @@ describe('HostTabComponent', () => {
                     nextServer: '192.0.2.2',
                     serverHostname: 'my-server',
                     bootFileName: '/tmp/bootfile1',
+                    hostname: 'mouse.example.org',
+                    ipReservations: [
+                        {
+                            address: '192.0.2.123',
+                        },
+                    ],
                 },
             ],
         }
@@ -167,7 +181,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should display v6 host information', () => {
-        const host = {
+        const host: Partial<Host> = {
             id: 1,
             hostIdentifiers: [
                 {
@@ -203,11 +217,41 @@ describe('HostTabComponent', () => {
                     appId: 1,
                     appName: 'frog',
                     dataSource: 'config',
+                    hostname: 'mouse.example.org',
+                    ipReservations: [
+                        {
+                            address: '2001:db8:1::1',
+                        },
+                        {
+                            address: '2001:db8:1::2',
+                        },
+                        {
+                            address: '2001:db8:2::/64',
+                        },
+                        {
+                            address: '2001:db8:3::/64',
+                        },
+                    ],
                 },
                 {
                     appId: 2,
                     appName: 'mouse',
                     dataSource: 'api',
+                    hostname: 'mouse.example.org',
+                    ipReservations: [
+                        {
+                            address: '2001:db8:1::1',
+                        },
+                        {
+                            address: '2001:db8:1::2',
+                        },
+                        {
+                            address: '2001:db8:2::/64',
+                        },
+                        {
+                            address: '2001:db8:3::/64',
+                        },
+                    ],
                 },
             ],
         }
@@ -285,7 +329,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should display lease information', () => {
-        const host = {
+        const host: Partial<Host> = {
             id: 1,
             hostIdentifiers: [
                 {
@@ -320,11 +364,39 @@ describe('HostTabComponent', () => {
                     appId: 1,
                     appName: 'frog',
                     dataSource: 'config',
+                    ipReservations: [
+                        {
+                            address: '2001:db8:1::1',
+                        },
+                        {
+                            address: '2001:db8:1::2',
+                        },
+                        {
+                            address: '2001:db8:2::/64',
+                        },
+                        {
+                            address: '2001:db8:3::/64',
+                        },
+                    ],
                 },
                 {
                     appId: 2,
                     appName: 'mouse',
                     dataSource: 'api',
+                    ipReservations: [
+                        {
+                            address: '2001:db8:1::1',
+                        },
+                        {
+                            address: '2001:db8:1::2',
+                        },
+                        {
+                            address: '2001:db8:2::/64',
+                        },
+                        {
+                            address: '2001:db8:3::/64',
+                        },
+                    ],
                 },
             ],
         }
@@ -359,9 +431,9 @@ describe('HostTabComponent', () => {
         expect(dhcpApi.getLeases).toHaveBeenCalled()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(5)
+        expect(fieldsets.length).toBe(6)
 
-        const ipReservationsFieldset = fieldsets[2]
+        const ipReservationsFieldset = fieldsets[3]
         expect(ipReservationsFieldset).toBeTruthy()
         const ipReservationTable = ipReservationsFieldset.query(By.css('table'))
         expect(ipReservationTable).toBeTruthy()
@@ -394,7 +466,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should display multiple lease information', () => {
-        const host = {
+        const host: Partial<Host> = {
             id: 1,
             hostIdentifiers: [
                 {
@@ -414,11 +486,21 @@ describe('HostTabComponent', () => {
                     appId: 1,
                     appName: 'frog',
                     dataSource: 'config',
+                    ipReservations: [
+                        {
+                            address: '192.0.2.1',
+                        },
+                    ],
                 },
                 {
                     appId: 2,
                     appName: 'mouse',
                     dataSource: 'api',
+                    ipReservations: [
+                        {
+                            address: '192.0.2.1',
+                        },
+                    ],
                 },
             ],
         }
@@ -455,9 +537,9 @@ describe('HostTabComponent', () => {
         expect(dhcpApi.getLeases).toHaveBeenCalled()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(5)
+        expect(fieldsets.length).toBe(6)
 
-        let ipReservationsFieldset = fieldsets[2]
+        let ipReservationsFieldset = fieldsets[3]
         expect(ipReservationsFieldset).toBeTruthy()
         let ipReservationTable = ipReservationsFieldset.query(By.css('table'))
         expect(ipReservationTable).toBeTruthy()
@@ -484,9 +566,9 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(5)
+        expect(fieldsets.length).toBe(6)
 
-        ipReservationsFieldset = fieldsets[2]
+        ipReservationsFieldset = fieldsets[3]
         expect(ipReservationsFieldset).toBeTruthy()
         ipReservationTable = ipReservationsFieldset.query(By.css('table'))
         expect(ipReservationTable).toBeTruthy()
@@ -801,7 +883,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should display different host data for different servers separately', () => {
-        const host = {
+        const host: Partial<Host> = {
             id: 1,
             hostIdentifiers: [
                 {
@@ -809,9 +891,23 @@ describe('HostTabComponent', () => {
                     idHexValue: '51:52:53:54',
                 },
             ],
-            addressReservations: [],
-            prefixReservations: [],
-            hostname: '',
+            addressReservations: [
+                {
+                    address: '2001:db8:1::1',
+                },
+                {
+                    address: '2001:db8:1::2',
+                },
+            ],
+            prefixReservations: [
+                {
+                    address: '2001:db8:2::/80',
+                },
+                {
+                    address: '2001:db8:3::/80',
+                },
+            ],
+            hostname: 'foo.example.org',
             subnetId: 1,
             subnetPrefix: '2001:db8:1::/64',
             localHosts: [
@@ -824,6 +920,7 @@ describe('HostTabComponent', () => {
                     nextServer: '192.0.2.1',
                     serverHostname: 'myhostname',
                     bootFileName: '/tmp/boot1',
+                    hostname: 'foo.example.org',
                     options: [
                         {
                             code: 1024,
@@ -833,6 +930,20 @@ describe('HostTabComponent', () => {
                         },
                     ],
                     optionsHash: '1111',
+                    ipReservations: [
+                        {
+                            address: '2001:db8:1::1',
+                        },
+                        {
+                            address: '2001:db8:1::2',
+                        },
+                        {
+                            address: '2001:db8:2::/80',
+                        },
+                        {
+                            address: '2001:db8:3::/80',
+                        },
+                    ],
                 },
                 {
                     appId: 2,
@@ -843,6 +954,7 @@ describe('HostTabComponent', () => {
                     nextServer: '192.0.2.2',
                     serverHostname: 'yourhostname',
                     bootFileName: '/tmp/boot2',
+                    hostname: 'bar.example.org',
                     options: [
                         {
                             code: 1024,
@@ -852,6 +964,14 @@ describe('HostTabComponent', () => {
                         },
                     ],
                     optionsHash: '2222',
+                    ipReservations: [
+                        {
+                            address: '2001:db8:1::3',
+                        },
+                        {
+                            address: '2001:db8:4::/80',
+                        },
+                    ],
                 },
             ],
         }
@@ -859,16 +979,20 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(8)
+        expect(fieldsets.length).toBe(12)
 
-        expect(fieldsets[2].properties.innerText).toContain('Boot Fields')
-        expect(fieldsets[3].properties.innerText).toContain('Boot Fields')
-        expect(fieldsets[4].properties.innerText).toContain('Client Classes')
-        expect(fieldsets[5].properties.innerText).toContain('Client Classes')
-        expect(fieldsets[6].properties.innerText).toContain('DHCP Options')
-        expect(fieldsets[7].properties.innerText).toContain('DHCP Options')
+        expect(fieldsets[2].properties.innerText).toContain('Hostname')
+        expect(fieldsets[3].properties.innerText).toContain('Hostname')
+        expect(fieldsets[4].properties.innerText).toContain('IP Reservations')
+        expect(fieldsets[5].properties.innerText).toContain('IP Reservations')
+        expect(fieldsets[6].properties.innerText).toContain('Boot Fields')
+        expect(fieldsets[7].properties.innerText).toContain('Boot Fields')
+        expect(fieldsets[8].properties.innerText).toContain('Client Classes')
+        expect(fieldsets[9].properties.innerText).toContain('Client Classes')
+        expect(fieldsets[10].properties.innerText).toContain('DHCP Options')
+        expect(fieldsets[11].properties.innerText).toContain('DHCP Options')
 
-        for (let i = 2; i < 8; i++) {
+        for (let i = 2; i < 12; i++) {
             let link = fieldsets[i].query(By.css('a'))
             expect(link).toBeTruthy()
             if (i % 2 === 0) {
@@ -882,7 +1006,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should display the same host data for different servers in one panel', () => {
-        const host = {
+        const host: Partial<Host> = {
             id: 1,
             hostIdentifiers: [
                 {
@@ -890,9 +1014,17 @@ describe('HostTabComponent', () => {
                     idHexValue: '51:52:53:54',
                 },
             ],
-            addressReservations: [],
-            prefixReservations: [],
-            hostname: '',
+            addressReservations: [
+                {
+                    address: '192.0.2.1',
+                },
+            ],
+            prefixReservations: [
+                {
+                    address: '192.0.0.128/30',
+                },
+            ],
+            hostname: 'foo.example.com',
             subnetId: 1,
             subnetPrefix: '192.0.2.0/24',
             localHosts: [
@@ -905,6 +1037,7 @@ describe('HostTabComponent', () => {
                     serverHostname: 'my-server',
                     bootFileName: '/tmp/boot1',
                     clientClasses: ['foo', 'bar'],
+                    hostname: 'foo.example.com',
                     options: [
                         {
                             code: 1024,
@@ -914,6 +1047,14 @@ describe('HostTabComponent', () => {
                         },
                     ],
                     optionsHash: '1111',
+                    ipReservations: [
+                        {
+                            address: '192.0.2.1',
+                        },
+                        {
+                            address: '192.0.0.128/30',
+                        },
+                    ],
                 },
                 {
                     appId: 2,
@@ -924,6 +1065,7 @@ describe('HostTabComponent', () => {
                     serverHostname: 'my-server',
                     bootFileName: '/tmp/boot1',
                     clientClasses: ['foo', 'bar'],
+                    hostname: 'foo.example.com',
                     options: [
                         {
                             code: 1024,
@@ -933,6 +1075,14 @@ describe('HostTabComponent', () => {
                         },
                     ],
                     optionsHash: '1111',
+                    ipReservations: [
+                        {
+                            address: '192.0.2.1',
+                        },
+                        {
+                            address: '192.0.0.128/30',
+                        },
+                    ],
                 },
             ],
         }
@@ -940,16 +1090,16 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         const fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(5)
+        expect(fieldsets.length).toBe(7)
 
-        expect(fieldsets[2].properties.innerText).toContain('Boot Fields')
-        expect(fieldsets[2].properties.innerText).toContain('All Servers')
-
-        expect(fieldsets[3].properties.innerText).toContain('Client Classes')
-        expect(fieldsets[3].properties.innerText).toContain('All Servers')
-
-        expect(fieldsets[4].properties.innerText).toContain('DHCP Options')
+        expect(fieldsets[4].properties.innerText).toContain('Boot Fields')
         expect(fieldsets[4].properties.innerText).toContain('All Servers')
+
+        expect(fieldsets[5].properties.innerText).toContain('Client Classes')
+        expect(fieldsets[5].properties.innerText).toContain('All Servers')
+
+        expect(fieldsets[6].properties.innerText).toContain('DHCP Options')
+        expect(fieldsets[6].properties.innerText).toContain('All Servers')
     })
 
     it('should display DHCP options panel for host with one daemon', () => {
@@ -990,14 +1140,14 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(5)
+        expect(fieldsets.length).toBe(7)
 
-        expect(fieldsets[2].properties.innerText).toContain('Boot Fields')
-        expect(fieldsets[2].properties.innerText).toContain('All Servers')
-        expect(fieldsets[3].properties.innerText).toContain('Client Classes')
-        expect(fieldsets[3].properties.innerText).toContain('All Servers')
-        expect(fieldsets[4].properties.innerText).toContain('DHCP Options')
+        expect(fieldsets[4].properties.innerText).toContain('Boot Fields')
         expect(fieldsets[4].properties.innerText).toContain('All Servers')
+        expect(fieldsets[5].properties.innerText).toContain('Client Classes')
+        expect(fieldsets[5].properties.innerText).toContain('All Servers')
+        expect(fieldsets[6].properties.innerText).toContain('DHCP Options')
+        expect(fieldsets[6].properties.innerText).toContain('All Servers')
     })
 
     it('should display a message about no client classes configured', () => {
@@ -1027,8 +1177,8 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(4)
-        expect(fieldsets[2].properties.innerText).toContain('No client classes configured.')
+        expect(fieldsets.length).toBe(6)
+        expect(fieldsets[4].properties.innerText).toContain('No client classes configured.')
     })
 
     it('should display dashes when boot fields are not specified', () => {
@@ -1059,10 +1209,10 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(5)
-        expect(fieldsets[2].properties.innerText).toContain('Next server\n0.0.0.0')
-        expect(fieldsets[2].properties.innerText).toContain('Server hostname\n-')
-        expect(fieldsets[2].properties.innerText).toContain('Boot file name\n-')
+        expect(fieldsets.length).toBe(7)
+        expect(fieldsets[4].properties.innerText).toContain('Next server\n0.0.0.0')
+        expect(fieldsets[4].properties.innerText).toContain('Server hostname\n-')
+        expect(fieldsets[4].properties.innerText).toContain('Boot file name\n-')
     })
 
     it('should display a message about no DHCP options configured', () => {
@@ -1098,8 +1248,8 @@ describe('HostTabComponent', () => {
         fixture.detectChanges()
 
         let fieldsets = fixture.debugElement.queryAll(By.css('p-fieldset'))
-        expect(fieldsets.length).toBe(4)
-        expect(fieldsets[3].properties.innerText).toContain('No options configured.')
+        expect(fieldsets.length).toBe(6)
+        expect(fieldsets[5].properties.innerText).toContain('No options configured.')
     })
 
     it('should group local hosts by app ID properly', () => {
@@ -1264,7 +1414,7 @@ describe('HostTabComponent', () => {
     })
 
     it('should group local hosts by different hash of DHCP options properly', () => {
-        const host = {
+        const host: Partial<Host> = {
             localHosts: [
                 {
                     appId: 1,
@@ -1292,7 +1442,7 @@ describe('HostTabComponent', () => {
                     optionsHash: 'foo',
                 },
             ],
-        } as Host
+        }
 
         component.host = host
         const groups = component.localHostsGroups.dhcpOptions
@@ -1316,6 +1466,125 @@ describe('HostTabComponent', () => {
         }
     })
 
+    it('should group local hosts by different IP reservations', () => {
+        const host: Partial<Host> = {
+            localHosts: [
+                {
+                    appId: 1,
+                    daemonId: 1,
+                    ipReservations: [
+                        {
+                            address: '10.0.0.1',
+                        },
+                        {
+                            address: '10.0.0.2',
+                        },
+                    ],
+                },
+                {
+                    appId: 1,
+                    daemonId: 2,
+                    ipReservations: [
+                        {
+                            address: '10.0.0.1',
+                        },
+                        {
+                            address: '10.0.0.2',
+                        },
+                    ],
+                },
+                {
+                    appId: 2,
+                    daemonId: 3,
+                    ipReservations: [
+                        {
+                            address: '10.0.0.3',
+                        },
+                    ],
+                },
+                {
+                    appId: 2,
+                    daemonId: 4,
+                    ipReservations: [
+                        {
+                            address: '10.0.0.3',
+                        },
+                        {
+                            address: '10.0.0.4',
+                        },
+                    ],
+                },
+            ],
+        }
+
+        component.host = host
+        const groups = component.localHostsGroups.ipReservations
+
+        expect(groups.length).toBe(3)
+
+        for (let group of groups) {
+            const appId = group[0].appId
+
+            switch (appId) {
+                case 1:
+                    expect(group.length).toBe(2)
+                    expect(group[0].daemonId).toBeLessThanOrEqual(2)
+                    break
+                case 2:
+                    expect(group.length).toBe(1)
+                    expect(group[0].daemonId).toBeGreaterThanOrEqual(3)
+                    expect(group[0].daemonId).toBeLessThanOrEqual(4)
+            }
+        }
+    })
+
+    it('should group local hosts by different hostnames', () => {
+        const host: Partial<Host> = {
+            hostname: 'ignored',
+            localHosts: [
+                {
+                    appId: 1,
+                    daemonId: 1,
+                    hostname: 'foo',
+                },
+                {
+                    appId: 1,
+                    daemonId: 2,
+                    hostname: 'foo',
+                },
+                {
+                    appId: 2,
+                    daemonId: 3,
+                    hostname: 'bar',
+                },
+                {
+                    appId: 2,
+                    daemonId: 4,
+                },
+            ],
+        }
+
+        component.host = host
+        const groups = component.localHostsGroups.hostname
+
+        expect(groups.length).toBe(3)
+
+        for (let group of groups) {
+            const appId = group[0].appId
+
+            switch (appId) {
+                case 1:
+                    expect(group.length).toBe(2)
+                    expect(group[0].daemonId).toBeLessThanOrEqual(2)
+                    break
+                case 2:
+                    expect(group.length).toBe(1)
+                    expect(group[0].daemonId).toBeGreaterThanOrEqual(3)
+                    expect(group[0].daemonId).toBeLessThanOrEqual(4)
+            }
+        }
+    })
+
     it('should group all local hosts into a single group if there are no differences', () => {
         const host = {
             localHosts: [
@@ -1348,6 +1617,8 @@ describe('HostTabComponent', () => {
             component.localHostsGroups.bootFields,
             component.localHostsGroups.clientClasses,
             component.localHostsGroups.dhcpOptions,
+            component.localHostsGroups.hostname,
+            component.localHostsGroups.ipReservations,
         ]) {
             expect(groups.length).toBe(1)
             const group = groups[0]
